<?xml version="1.0"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
                 version="1.0"
                 xmlns:UML="org.omg.xmi.namespace.UML"
                 xmlns:redirect="http://xml.apache.org/xalan/redirect"
                 extension-element-prefixes="redirect">

  <xsl:output method="text" indent="yes" />         
  
  <xsl:key name="classifier" match="//UML:Class |//UML:Interface |//UML:DataType |//UML:Stereotype" use="@xmi.id" /> 
  <xsl:key name="generalization" match="//UML:Generalization" use="@xmi.id" /> 
  <xsl:key name="multiplicity" match="//UML:Multiplicity" use="@xmi.id" /> 
  <xsl:key name="stereotype" match="//UML:Stereotype" use="@xmi.id" /> 

<xsl:variable name="newline">
<xsl:text>&#10;</xsl:text>
</xsl:variable>


<!-- Document Root -->
<xsl:template match="/">
    <!-- Model Name in window title -->
    <!-- Model Name in document heading -->
		    <!-- Classes -->
<h1><xsl:text>Generated Class Files: </xsl:text></h1>
		    <xsl:apply-templates select="//UML:Class[@xmi.id]">
		        <xsl:sort select="@name"/>
		    </xsl:apply-templates>
</xsl:template>


<!-- Class -->
<xsl:template match="UML:Class">
    <xsl:variable name="element_name" select="@name" /> 
    <xsl:variable name="xmi_id" select="@xmi.id" />
    <xsl:variable name="file_name" select="concat($element_name,'.pvc')" />
<xsl:value-of select="$newline"/>
<xsl:value-of select="$element_name"/>
<BR />
   <redirect:write select="$file_name">
! Definition of Class<xsl:value-of select="$element_name"/>
<xsl:value-of select="$newline"/>
		 <xsl:for-each select="UML:ModelElement.comment/UML:Comment">
		        <xsl:call-template name="UMLComment"/>
		 </xsl:for-each>  
                <xsl:text>DEF CLASS </xsl:text><xsl:text>"</xsl:text><xsl:value-of select="@name"/><xsl:text>"</xsl:text>
				    <xsl:apply-templates select="UML:ModelElement.stereotype" /> 
					<xsl:call-template name="supertypes">
						  <xsl:with-param name="source" select="$xmi_id" /> 
					</xsl:call-template>
			<xsl:value-of select="$newline"/>
            <xsl:call-template name="attributes"/> 
			<xsl:value-of select="$newline"/>
			<xsl:call-template name="associations">
			  <xsl:with-param name="source" select="$xmi_id" /> 
			</xsl:call-template>
			<xsl:value-of select="$newline"/>
           <xsl:call-template name="operations"/>
<xsl:value-of select="$newline"/>
<xsl:text>END DEF</xsl:text>
    </redirect:write>
</xsl:template>
<!-- eo class template -->

<xsl:template match="UML:ModelElement.stereotype">
    		<xsl:for-each select="UML:Stereotype">
		       <xsl:variable name="target" select="@xmi.idref"/>
				<xsl:call-template name="classify">
		           <xsl:with-param name="target" select="$target"/>
			    </xsl:call-template>
		    </xsl:for-each>       
</xsl:template>


<xsl:template name="UMLComment">
    <xsl:variable name="id" select="@xmi.idref"/>
    <xsl:for-each select="/XMI/XMI.content/UML:Model/UML:Namespace.ownedElement/descendant::*[@xmi.id = $id]">
      <xsl:call-template name="comment"/> 
    </xsl:for-each>
</xsl:template>

  <xsl:template name="comment">
<xsl:text>! </xsl:text>
	  <xsl:value-of select="@body"/>  
<xsl:value-of select="$newline"/>	  
  </xsl:template>


<!-- Supertypes (inheritance) -->
<xsl:template name="supertypes">
     <xsl:param name="source" /> 
    
    <!-- Generalizations identify supertypes -->
	 <xsl:variable name="generalizations" select="//UML:Generalization[UML:Generalization.child/*/@xmi.idref=$source]" />
    <xsl:if test="count($generalizations) > 0">
<xsl:value-of select="$newline"/>
       <xsl:text>LIKE </xsl:text>
            <xsl:for-each select="$generalizations">
                <!-- get the parent in the generalization                 -->
                <xsl:variable name="target" 
                   select="UML:Generalization.parent/*/@xmi.idref" />  
	                <xsl:text>"</xsl:text>
                  <xsl:call-template name="classify">
                    <xsl:with-param name="target" select="$target"/>
                </xsl:call-template>
	                <xsl:text>"</xsl:text>
                <xsl:if test="position() != last()">
                    <xsl:text>,  </xsl:text>
                </xsl:if>
            </xsl:for-each>
    </xsl:if>  
</xsl:template>
<!-- eo supertypes template -->


<!-- Classification -->
<xsl:template name="classify">
    <xsl:param name="target"/>

    <xsl:variable name="classifier" select="key('classifier', $target)" />
    <xsl:variable name="classifier_name" select="$classifier/@name" />
    <xsl:variable name="type" select="name($classifier)" />

    
    <!-- Get the type of the classifier (class, interface, datatype) -->
    <xsl:variable name="classifier_type">
    <xsl:choose>
        <xsl:when test="$type='UML:Class'">classifier</xsl:when>
        <xsl:when test="$type='UML:Interface'">interface</xsl:when>
        <xsl:when test="$type='UML:DataType'">datatype</xsl:when>
        <xsl:otherwise>classifier</xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    
    <xsl:choose>
        <!-- Datatypes don't have hyperlinks -->
        <xsl:when test="$type='UML:DataType'">
                <xsl:value-of select="$classifier_name"/>
        </xsl:when>
        <xsl:when test="$type='UML:Stereotype'">
            <xsl:if test="string($classifier_name) = 'Singleton'">
				<xsl:text> UNIQUE </xsl:text>
            </xsl:if>      
            <xsl:if test="string($classifier_name) = 'singleton'">
				<xsl:text> UNIQUE </xsl:text>
            </xsl:if>      
        </xsl:when>
        
        <!-- Classes and Interfaces have hyperlinks -->
        <!-- The classifier type is used to style appropriately -->
        <xsl:otherwise>
<xsl:value-of select="$classifier_name"/>
        </xsl:otherwise>
    </xsl:choose>

</xsl:template>
<!-- eo classify template -->

<!-- Attributes -->
<xsl:template name="attributes">
! Attributes:<xsl:value-of select="$newline"/>
<xsl:apply-templates select="UML:Classifier.feature/UML:Attribute" />
</xsl:template>

<!-- eo attrb calls below -->
<xsl:template match="UML:Attribute">
    <xsl:variable name="target" select='@xmi.id'/>
       <xsl:variable name="visiblity" select="@visibility"/>
        <xsl:choose>
            <xsl:when test="string($visiblity) = 'private'">
<xsl:text>LOCAL  </xsl:text>
            </xsl:when>
            <xsl:otherwise>
<xsl:text>PROPERTY  </xsl:text>
            </xsl:otherwise>
        </xsl:choose> 
			    <xsl:variable name="element_name" select="@name"/>
		         <xsl:value-of select="$element_name"/>
    <xsl:apply-templates select="UML:Attribute.initialValue" />
    <xsl:apply-templates select="UML:StructuralFeature.type/UML:Class" />    
        		<xsl:for-each select="UML:ModelElement.comment/UML:Comment">
		        	<xsl:call-template name="UMLComment"/>
			    </xsl:for-each>        
<xsl:value-of select="$newline"/>			    
</xsl:template>
<!-- eo attrib helper method -->

<xsl:template match="UML:StructuralFeature.type/UML:Class">
        <xsl:variable name="target" select='@xmi.idref'/>
<xsl:text> ! INSTANCEOF: </xsl:text>
        <xsl:call-template name="classify">
            <xsl:with-param name="target" select="$target" />
        </xsl:call-template>
</xsl:template>

 <!--  Associations   --> 
	 <xsl:template name="associations">
	  <xsl:param name="source" /> 
	  <xsl:variable name="association" select="//UML:Association.connection [UML:AssociationEnd/UML:AssociationEnd.participant/*/ @xmi.idref=$source]" /> 
	 <xsl:if test="count($association) > 0">
! Associations:
		<xsl:value-of select="$newline"/>
<xsl:for-each select="$association">
<xsl:call-template name="association_end">
<xsl:with-param name="source" select="$source" /> 
</xsl:call-template>
<xsl:value-of select="$newline"/>
	  </xsl:for-each>
	  </xsl:if>
	  </xsl:template>
 <!--  Associations end  --> 

 <!--  Association Ends  --> 
	 <xsl:template name="association_end">
	  <xsl:param name="source" /> 
	  <xsl:variable name="myend" select="./UML:AssociationEnd [UML:AssociationEnd.participant/*/@xmi.idref=$source]" /> 
	  <xsl:variable name="farend" select="./UML:AssociationEnd [UML:AssociationEnd.participant/*/@xmi.idref!=$source]" /> 
	  <xsl:variable name="endpt" select="$farend/UML:AssociationEnd.participant" /> 
	  <xsl:variable name="endpName" select="$endpt/*/@xmi.idref" /> 
      <xsl:variable name="classifier" select="key('classifier', $endpName)" />
      <xsl:variable name="classifier_name" select="$classifier/@name" />
      <xsl:if test="string-length($classifier_name) > 0"> 
      <xsl:variable name="navigbl" select="$farend/@isNavigable"/>
	 <!--  test if this is a navigable association  --> 
     <xsl:if test="string($navigbl) = 'true'">
		 <!--  Visibility  --> 
		 <!--  Properties  --> 
		 <!--  Participant --> 
		 <!-- if there's no attrb name should use Participant name -->
						 <xsl:choose>
							 <xsl:when test="string-length($myend/@visibility) > 0">
							       <xsl:variable name="visiblity" select="$myend/@visibility"/>
							        <xsl:choose>
							            <xsl:when test="string($visiblity) = 'private'">
<xsl:text>LOCAL  </xsl:text>
							            </xsl:when>
							            <xsl:otherwise>
<xsl:text>PROPERTY  </xsl:text>
							            </xsl:otherwise>
			         </xsl:choose> 
						  </xsl:when>
						 <xsl:otherwise>
<xsl:text disable-output-escaping="yes">LOCAL </xsl:text> 
						  </xsl:otherwise>
		     		  </xsl:choose>
                <xsl:apply-templates select="UML:AssociationEnd" />
		     		  
						 <xsl:call-template name="classify">
						  <xsl:with-param name="target" select="$endpt/*/@xmi.idref" /> 
						  </xsl:call-template>
	  </xsl:if>
     </xsl:if>
  </xsl:template> 

<xsl:template match="UML:Attribute.initialValue">
    <xsl:variable name="target" select='@xmi.id'/>
    <xsl:apply-templates select="UML:Expression" />
</xsl:template>

<xsl:template match="UML:Expression">
         <xsl:value-of select="target"/>
    		<xsl:text> = </xsl:text>
    		 <xsl:value-of select="@body"/>
</xsl:template>


<xsl:template match="UML:AssociationEnd">
      <xsl:variable name="end_name" select="@name" />
       <xsl:choose>
           <xsl:when test="string-length($end_name) > 0">
			      <xsl:value-of select="@name"/>
			    <xsl:text disable-output-escaping="yes">	! INSTANCEOF: </xsl:text> 
           </xsl:when>
           <xsl:otherwise>
           </xsl:otherwise>      
       </xsl:choose> 
</xsl:template>


<!-- Operations -->
<xsl:template name="operations">
! Operations:
<xsl:value-of select="$newline"/>

<xsl:apply-templates select="UML:Classifier.feature/UML:Operation" />
</xsl:template>


<xsl:template match="UML:Operation">
    <xsl:variable name="parameters"
         select="UML:BehavioralFeature.parameter/UML:Parameter" />
<xsl:text>! </xsl:text>         <xsl:value-of select="$newline"/>
<xsl:text>! </xsl:text><xsl:value-of select="@name"/>
<xsl:for-each select="UML:ModelElement.comment/UML:Comment">
<xsl:value-of select="$newline"/>
<xsl:call-template name="UMLComment"/><xsl:value-of select="$newline"/>
		      </xsl:for-each>    
<xsl:value-of select="$newline"/>
		       <xsl:variable name="visiblity" select="@visibility" />
		        <xsl:choose>
		            <xsl:when test="string($visiblity) = 'private'">
						<xsl:text>FUNCTION LOCAL </xsl:text>
		            </xsl:when>
		            <xsl:otherwise>
						<xsl:text>FUNCTION </xsl:text>
		            </xsl:otherwise>
		        </xsl:choose>
<!-- get function name -->
<xsl:value-of select="@name"/>
<!-- get args  -->
<xsl:text>( </xsl:text><xsl:apply-templates select="$parameters" /><xsl:text> )</xsl:text>
   <xsl:variable name="parameter-count" select="count($parameters)" />
    <!-- return is included in parameter count so if only one parameter then really no args for function  -->
    <xsl:if test="not(normalize-space($parameter-count)='1')">
<!-- get args  -->
<xsl:value-of select="$newline"/><xsl:text>ENTER </xsl:text><xsl:apply-templates select="$parameters" />
			<xsl:value-of select="$newline"/>
    </xsl:if >
! TODO: Implement Function    
		<xsl:value-of select="$newline"/>
RETURN
<xsl:value-of select="$newline"/>
</xsl:template>

<!-- Parameter -->
<xsl:template match="UML:Parameter">
<xsl:variable name="param_name" select="@name"/>
	<!-- skip the return parameter -->
<xsl:if test="string($param_name) != 'return'">
<xsl:variable name="target" select="UML:Parameter.type/*/@xmi.idref" />
<xsl:value-of select="@name"/>
<xsl:if test="position() != last()">
<xsl:text>,  </xsl:text>
</xsl:if>
</xsl:if>
</xsl:template>



</xsl:stylesheet>