<?xml version="1.0"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml" xmlns:UML="org.omg.xmi.namespace.UML" version="1.0" exclude-result-prefixes="#default">
  <xsl:output method="html" indent="yes" doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN" /> 
  
  <xsl:key name="classifier" match="//UML:Class |//UML:Interface |//UML:DataType |//UML:Stereotype" use="@xmi.id" /> 
  <xsl:key name="generalization" match="//UML:Generalization" use="@xmi.id" /> 
  <xsl:key name="multiplicity" match="//UML:Multiplicity" use="@xmi.id" /> 
  <xsl:key name="stereotype" match="//UML:Stereotype" use="@xmi.id" /> 



<!-- Document Root -->
<xsl:template match="/">
    <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"> 
    <head>
<!-- Model Name in window title -->
    <xsl:apply-templates select="//UML:Model[@xmi.id]" mode="head" /> 
    <link href="xmi.css" rel="stylesheet" type="text/css"/>
    </head>
    <body LINK="#008000" VLINK="#008000">
      <!-- Model Name in document heading -->
      <xsl:apply-templates select="//UML:Model[@xmi.id]" mode="body"/>
	<FONT COLOR="#0000FF">!-- ProvideXMI Generated Classes Begin --</FONT>
    <!-- Classes -->
    <xsl:apply-templates select="//UML:Class[@xmi.id]">
        <xsl:sort select="@name"/>
    </xsl:apply-templates>


    </body>
    </html>

</xsl:template>


<!-- Window Title -->
<xsl:template match="UML:Model" mode="head">
    <title><xsl:value-of select="@name" /></title>
</xsl:template>


<!-- Document Heading -->
<xsl:template match="UML:Model" mode="body">
  <left><h1><xsl:value-of select="@name"/></h1></left>
</xsl:template>

<!-- Class -->
<xsl:template match="UML:Class">
    <xsl:variable name="element_name" select="@name"/>
    <xsl:variable name="xmi_id" select="@xmi.id" />
    <div align="left">
    <table border="1" width="100%" cellpadding="0" >
    	<tr>
    	<td>
		    <table border="0" width="100%" cellpadding="0" >
		    <tr>
		        <td class="info-title" ><FONT COLOR="#0000FF">! Definition of Class
		        <!-- create a hyperlink target for the name -->
		            <a name="{$element_name}">
		                <xsl:value-of select="$element_name"/>
		            </a>
		            </FONT>
				</td>            
		    </tr>
		    <tr>
		        <td class="info" ><FONT COLOR="#0000FF">! 
		     <xsl:for-each select="UML:ModelElement.comment/UML:Comment">
		        <xsl:call-template name="UMLComment"/>
		      </xsl:for-each>    
				</FONT></td>            
		    </tr>
		    <tr>
		        <td class="pvx-directive" ><FONT COLOR="#000000">
		                <xsl:text>DEF CLASS </xsl:text></FONT>
						 	<FONT COLOR="#008000">
						 <span class="class-name">
							<xsl:text>"</xsl:text><xsl:value-of select="@name"/><xsl:text>"</xsl:text>
						 </span>                
						 </FONT>
						    <xsl:apply-templates select="UML:ModelElement.stereotype" /> 
		        </td>
		    </tr>
		
					<xsl:call-template name="supertypes">
						  <xsl:with-param name="source" select="$xmi_id" /> 
					</xsl:call-template>
		    </table>

    </td >
    </tr>
    <tr>
    <td >
        <table width="100%" border="0" cellpadding="0" cellspacing="0">
        
            <xsl:call-template name="attributes"/> 
        
			<xsl:call-template name="associations">
			  <xsl:with-param name="source" select="$xmi_id" /> 
			</xsl:call-template>
        </table>
    </td>
	</tr>        
    <tr>
    <td >
        <table width="100%" border="0" cellpadding="0" cellspacing="0">
            <xsl:call-template name="operations"/>
		    <tr>
		    <td >
						<xsl:text>END DEF</xsl:text>
		    </td>
		    </tr>
        </table>
    </td>
    </tr>

    </table>
    </div>
    
		                <p class="info"><FONT COLOR="#0000FF">!-- End of class definition --</FONT></p>
</xsl:template>
<!-- eo class template -->



<xsl:template match="UML:ModelElement.stereotype">
    		<xsl:for-each select="UML:Stereotype">
		       <xsl:variable name="target" select="@xmi.idref"/>
				<xsl:call-template name="classify">
		           <xsl:with-param name="target" select="$target"/>
			    </xsl:call-template>
		    </xsl:for-each>       
</xsl:template>



<!-- eo stereo type template -->

<!-- Supertypes (inheritance) -->
<xsl:template name="supertypes">
     <xsl:param name="source" /> 
    
    <!-- Generalizations identify supertypes -->
	 <xsl:variable name="generalizations" select="//UML:Generalization[UML:Generalization.child/*/@xmi.idref=$source]" />
    <xsl:if test="count($generalizations) > 0">
        <tr>
            <td  class="pvx-directive"><FONT COLOR="#000000">LIKE 
            <xsl:for-each select="$generalizations">
            
                <!-- get the parent in the generalization                 -->
                <xsl:variable name="target" 
                   select="UML:Generalization.parent/*/@xmi.idref" />  
				 	<FONT COLOR="#008000">
	                <xsl:text>"</xsl:text>
	                
                  <xsl:call-template name="classify">
                    <xsl:with-param name="target" select="$target"/>
                </xsl:call-template>
                
	                <xsl:text>"</xsl:text>
				 	</FONT>
                
                <xsl:if test="position() != last()">
                    <xsl:text>,  </xsl:text>
                </xsl:if>
            </xsl:for-each>
            </FONT>
            </td>
        </tr>
    </xsl:if>  
</xsl:template>
<!-- eo supertypes template -->

<!-- Classification -->
<xsl:template name="classify">
    <xsl:param name="target"/>

    <xsl:variable name="classifier" select="key('classifier', $target)" />
    <xsl:variable name="classifier_name" select="$classifier/@name" />
    <xsl:variable name="type" select="name($classifier)" />
    <!-- Get the type of the classifier (class, interface, datatype) -->
    <xsl:variable name="classifier_type">
    <xsl:choose>
        <xsl:when test="$type='UML:Class'">classifier</xsl:when>
        <xsl:when test="$type='UML:Interface'">interface</xsl:when>
        <xsl:when test="$type='UML:DataType'">datatype</xsl:when>
        <xsl:otherwise>classifier</xsl:otherwise>
    </xsl:choose>
    </xsl:variable>
    
    <xsl:choose>
        <!-- Datatypes don't have hyperlinks -->
        <xsl:when test="$type='UML:DataType'">
            <span class="datatype">
				 	<FONT COLOR="#008000">
                <xsl:value-of select="$classifier_name"/>
				 	</FONT>
            </span>
        </xsl:when>
        <xsl:when test="$type='UML:Stereotype'">
            <xsl:if test="string($classifier_name) = 'Singleton'">
				<xsl:text> UNIQUE </xsl:text>
            </xsl:if>      
            <xsl:if test="string($classifier_name) = 'singleton'">
				<xsl:text> UNIQUE </xsl:text>
            </xsl:if>      
        </xsl:when>
        
        <!-- Classes and Interfaces have hyperlinks -->
        <!-- The classifier type is used to style appropriately -->
        <xsl:otherwise>
<!--            <xsl:if test="string-length($classifier) > 0"> -->
                <a class="{$classifier_type}" href="#{$classifier_name}">
                <xsl:value-of select="$classifier_name"/>
                </a>
<!--            </xsl:if> -->
        </xsl:otherwise>
    </xsl:choose>

</xsl:template>
<!-- eo classify template -->


<!-- Attributes -->
<xsl:template name="attributes">
    <tr>
        <td class="info-title"><FONT COLOR="#0000FF">!	Attributes:</FONT></td>
    </tr>
    <xsl:apply-templates select="UML:Classifier.feature/UML:Attribute" />
</xsl:template>

<!-- eo attrb calls below -->

<xsl:template match="UML:Attribute">
    <xsl:variable name="target" select='@xmi.id'/>
    <tr>
        <td class="pvx-directive">
       <xsl:variable name="visiblity" select="@visibility"/>
        <xsl:choose>
            <xsl:when test="string($visiblity) = 'private'">
				<xsl:text>LOCAL  </xsl:text>
            </xsl:when>
            <xsl:otherwise>
				<xsl:text>PROPERTY  </xsl:text>
            </xsl:otherwise>
            
        </xsl:choose>
				 	<FONT COLOR="#008000">
			    <xsl:variable name="element_name" select="@name"/>
		         <xsl:value-of select="$element_name"/>
 				 	</FONT>
    <xsl:apply-templates select="UML:Attribute.initialValue" />
        		<xsl:for-each select="UML:ModelElement.comment/UML:Comment">
		        	<xsl:call-template name="UMLComment"/>
			    </xsl:for-each>        

    <xsl:apply-templates select="UML:StructuralFeature.type/UML:Class" />
			    
        </td>
        
    </tr>
    
</xsl:template>
<!-- eo attrib helper method -->

<xsl:template match="UML:Attribute.initialValue">
    <xsl:variable name="target" select='@xmi.id'/>
    <xsl:apply-templates select="UML:Expression" />
</xsl:template>

<xsl:template match="UML:StructuralFeature.type/UML:Class">
        <xsl:variable name="target" select='@xmi.idref'/>
<FONT COLOR="#0000FF">
         <xsl:text> ! INSTANCEOF: </xsl:text>
        <xsl:call-template name="classify">
            <xsl:with-param name="target" select="$target" />
        </xsl:call-template>
</FONT>        
</xsl:template>


<xsl:template match="UML:Expression">
         <xsl:value-of select="target"/>
    		<xsl:text> = </xsl:text>
				 	<FONT COLOR="#008080">
    		 <xsl:value-of select="@body"/>
    		 	 	</FONT>
</xsl:template>

<!-- Operations -->
<xsl:template name="operations">
    <tr>
        <td class="info-title"><FONT COLOR="#0000FF">!	Operations:</FONT></td>
    </tr>
							     
    <xsl:apply-templates select="UML:Classifier.feature/UML:Operation" />
</xsl:template>


<xsl:template match="UML:Operation">
<!--    <xsl:variable name="parameters" select="UML:BehavioralFeature.parameter/UML:Parameter[UML:Parameter.kind/@xmi.value!='return']" /> -->
    <xsl:variable name="parameters"
         select="UML:BehavioralFeature.parameter/UML:Parameter" />

    <tr>
        <td class="info">
	 	<FONT COLOR="#0000FF">
		<xsl:text>! </xsl:text>
            <xsl:value-of select="@name"/>
		</FONT>						
        </td>
    </tr>
    <tr>
     <td class="info" ><FONT COLOR="#0000FF">! 
		     <xsl:for-each select="UML:ModelElement.comment/UML:Comment">
		        <xsl:call-template name="UMLComment"/>
		      </xsl:for-each>    
		</FONT></td>            
    </tr>
    
    <tr>
        <td class="pvx-directive"> 
		       <xsl:variable name="visiblity" select="@visibility" />
		        <xsl:choose>
		            <xsl:when test="string($visiblity) = 'private'">
						<xsl:text>FUNCTION LOCAL  </xsl:text>
		            </xsl:when>
		            <xsl:otherwise>
						<xsl:text>FUNCTION  </xsl:text>
		            </xsl:otherwise>
		            
		        </xsl:choose>

<!-- get function name -->

			<span class="feature-detail">
				 	<FONT COLOR="#008000">
            <xsl:value-of select="@name"/>
            				 	</FONT>
            </span>

<!-- get args  -->
		<span class="pvx-directive">(</span>
                <xsl:apply-templates select="$parameters" />
		<span class="pvx-directive">)</span>

  </td> 
        
    </tr>

    <xsl:variable name="parameter-count" select="count($parameters)" />

<!-- return is included in parameter count so if only one parameter then really no args for function  -->
    <xsl:if test="not(normalize-space($parameter-count)='1')">

    <tr>
<!-- get args  -->
        <td class="pvx-directive" valign="top">ENTER 
                <xsl:apply-templates select="$parameters" />
        </td>
    </tr>


    </xsl:if >
    <tr>
        <td class="pvx-directive">
			<xsl:text>RETURN</xsl:text>
		</td>
    </tr>

</xsl:template>

<!-- Parameter -->
<xsl:template match="UML:Parameter">
    <xsl:variable name="param_name" select="@name"/>

	<!-- skip the return parameter -->
     <xsl:if test="string($param_name) != 'return'">
       <xsl:variable name="target" select="UML:Parameter.type/*/@xmi.idref" />
		<span class="feature-detail" >
		<FONT COLOR="#008000">
		            <xsl:value-of select="@name"/>
						<xsl:if test="position() != last()">
		                    <xsl:text>,  </xsl:text>
		                </xsl:if>
		</FONT>                
		</span>                
     </xsl:if>
        
</xsl:template>


 <!--  Associations   --> 
	 <xsl:template name="associations">
	  <xsl:param name="source" /> 
	  <xsl:variable name="association" select="//UML:Association.connection [UML:AssociationEnd/UML:AssociationEnd.participant/*/ @xmi.idref=$source]" /> 
	 <xsl:if test="count($association) > 0">
	 <tr>
		  <td class="info"><FONT COLOR="#0000FF">! Associations:</FONT></td> 
     </tr>
		 <xsl:for-each select="$association">
    <!--		  <xsl:call-template name="descriptions" />  -->
		 <xsl:call-template name="association_end">
		  <xsl:with-param name="source" select="$source" /> 
		  </xsl:call-template>
	  </xsl:for-each>
	  </xsl:if>
	  </xsl:template>
 <!--  Association Ends  --> 

	 <xsl:template name="association_end">
	  <xsl:param name="source" /> 
	  <xsl:variable name="myend" select="./UML:AssociationEnd [UML:AssociationEnd.participant/*/@xmi.idref=$source]" /> 
	  <xsl:variable name="farend" select="./UML:AssociationEnd [UML:AssociationEnd.participant/*/@xmi.idref!=$source]" /> 
	  <xsl:variable name="endpt" select="$farend/UML:AssociationEnd.participant" /> 
	  <xsl:variable name="endpName" select="$endpt/*/@xmi.idref" /> 
      <xsl:variable name="classifier" select="key('classifier', $endpName)" />
      <xsl:variable name="classifier_name" select="$classifier/@name" />
	  
      <xsl:if test="string-length($classifier_name) > 0"> 

      <xsl:variable name="navigbl" select="$farend/@isNavigable"/>

	 <!--  test if this is a navigable association  --> 
     <xsl:if test="string($navigbl) = 'true'">
     
		 <!--  Visibility  --> 
			 <tr>
				 <td class="pvx-directive">
		 <!--  Properties  --> 
		 <!--  Participant --> 
		 <!-- if there's no attrb name should use Participant name -->
		 
				 <span class="feature-detail">
						 <xsl:choose>
							 <xsl:when test="string-length($myend/@visibility) > 0">
							       <xsl:variable name="visiblity" select="$myend/@visibility"/>
							        <xsl:choose>
							            <xsl:when test="string($visiblity) = 'private'">
											<xsl:text>LOCAL  </xsl:text>
							            </xsl:when>
							            <xsl:otherwise>
											<xsl:text>PROPERTY  </xsl:text>
							            </xsl:otherwise>
						         </xsl:choose> 
						  </xsl:when>
						 <xsl:otherwise>
								<xsl:text disable-output-escaping="yes">LOCAL </xsl:text> 
						  </xsl:otherwise>
		     		  </xsl:choose>
				
						<FONT COLOR="#008000">		 
	
						
                <xsl:apply-templates select="UML:AssociationEnd">
                </xsl:apply-templates>
                
						 <xsl:call-template name="classify">
						  <xsl:with-param name="target" select="$endpt/*/@xmi.idref" /> 
						  </xsl:call-template>
						  </FONT>
						  </span>
				 
				  </td>
			  </tr>
	  </xsl:if>
     </xsl:if>
  </xsl:template> 

<xsl:template match="UML:AssociationEnd">
      <xsl:variable name="end_name" select="@name" />
       <xsl:choose>
           <xsl:when test="string-length($end_name) > 0">
				<FONT COLOR="#008000">		 
			      <xsl:value-of select="@name"/>
    			  </FONT>
				<FONT COLOR="#0000FF">		 
			    <xsl:text disable-output-escaping="yes">	! INSTANCEOF: </xsl:text> 
			  </FONT>
           </xsl:when>
           <xsl:otherwise>
<!--               <xsl:apply-templates select="UML:AssociationEnd.participant/UML:Class" />  -->
           </xsl:otherwise>      
       </xsl:choose> 
</xsl:template>

<xsl:template match="UML:AssociationEnd.participant/UML:Class">      
	 <xsl:call-template name="classify">
	  <xsl:with-param name="target" select="@xmi.idref" /> 
	  </xsl:call-template>
</xsl:template>

<xsl:template name="UMLComment">
    <xsl:variable name="id">
      <xsl:value-of select="@xmi.idref"/>
    </xsl:variable>
    <xsl:for-each select="/XMI/XMI.content/UML:Model/UML:Namespace.ownedElement/descendant::*[@xmi.id = $id]">
      <xsl:call-template name="comment"/>
    </xsl:for-each>
</xsl:template>

  <xsl:template name="comment">
    <xsl:element name="comment">
	  <xsl:value-of select="@body"/>  
    </xsl:element>
  </xsl:template>

</xsl:stylesheet>